


TACHYON
{
										PUMPTIMER.fth
										
										
		UPDATED 20121128.1300 - BBR - fixed misplaced semi-colon in PUMPTIMER
		UPDATED 20121127.1500 - BBR - extensive overhaul of PUMPTASK
								PUMPTIMER and pinio
		CREATED 20121124.1800 - BB Riley, Underhill Center, VT  
		                        brianbr@wulfden.org

		TESTED  Tachyon Forth V21 121114.1330 Extend 121113.1200
}		
		
: pumptimer.fth	." Smart Shallow Well Pump Timer - 121128.1300"  ;



#P7		CONSTANT	#lcdpin		\  LCD port

#P16	CONSTANT	#selectsw	\  SELECT switch
#P17	CONSTANT	#runsw		\  RUN switch
#P18	CONSTANT	#fulltank	\  tank full indicator
#P20	CONSTANT	#lowbatry	\  low battery indicator
#P22	CONSTANT	#runpump	\  to SSR switching pump AC power
#P26	CONSTANT	#enc.a		\  A output from encoder
#P27	CONSTANT	#enc.b		\  B output from encoder

#120	CONSTANT	#maxdur		\  max value for duration
#15		CONSTANT	#maxon		\  max value for on period
#10		CONSTANT	#maxoff		\  max value for off period

#P9		CONSTANT	#PUMP		\ base pins of 3 LED groups
#P13	CONSTANT	#MAIN
0		CONSTANT	GREEN		\ iopin offsets
1		CONSTANT	RED
2		CONSTANT	YELLOW

LONG	oldBits,newBits,varptr,runningtime,totaltime

LONG	maxdata,duration,onperiod,offperiod

\  -------- Misc Hardware -------------------------------

: LED	( on/off basepin offset -- ) + PIN! ;

: PUMP 			( on/off -- ) #runpump PIN! ;

: LOWBATTERY?	( -- true/false ) #lowbatry PIN@ 0= ;

: TANKFULL?		( -- true/false ) #fulltank PIN@ IF
					FALSE
				ELSE
					2 ms #fulltank PIN@ IF 
						FALSE 
					ELSE 
						TRUE 
					THEN
				THEN
				150 ms
				;

: SELECTED?		( -- true/fals )
				#selectsw PIN@ IF
					FALSE
				ELSE
					2 ms #selectsw PIN@ IF 
						FALSE 
					ELSE 
						TRUE 
					THEN
				THEN
				150 ms
				;

: RUN?		( -- true/false )
			#runsw PIN@ IF
				FALSE
			ELSE
				2 ms #runsw PIN@ IF 
					FALSE 
				ELSE 
					TRUE 
				THEN
			THEN
			150 ms
			;



\  ------------- Quadrature Encoder --------------------

: QE@	( -- ) IN@ #enc.a SHR 3 AND ;	\ get current state	

: QDEC@	( dataddr datamax -- )	\ QuadratureDecoder
	maxdata !					\  write max allowed value
	varptr !					\  write var address to pointer
	#enc.a PININP				\  Set encoder pins to inputs
	#enc.b PININP
	QE@  oldBits !				\  Pre-initialize oldBits (prevents 
								\       non-zero starting count)
	BEGIN
    	QE@	DUP newBits !		\  read encoder
		oldBits @
		<> IF					\  compare oldBits to newBits. Has it changed? 
		 		\ counter = counter - 1 + (2 * (newBits.BIT0 ^ oldBits.BIT1))
	     		\ the next 4 lines are the TF implementation of the above PBASIC
			newBits @ 1 AND		\  determine direction and add or subtract from counter
			oldBits @ 2/
			XOR 2* 1-
			varptr @ @ +		\  add to old var 
			0 MAX				\  apply min/max - 0 minimum
			maxdata @ MIN		\  cap the top
			varptr @ !			\  write new val to var
			newBits @ oldBits !  \ Update oldBits
			." ?x09?y0" varptr @ @ $030A .NUM
		THEN

	SELECTED? UNTIL				
	DROP
	;

\  ---------------- LCD -------------------------------

	{ The "?" in the various string prints, is the escape character for 
	  Peter Anderson Serial LCD firmware }

: LCDEMIT	#lcdpin SEROUT ;
: LCD		' LCDEMIT uemit W! ;

: BACKLIGHT	( on/off -- ) IF ." ?BA0" ELSE ." ?B00" THEN ;	
										\  This must be used with console redirected
										\  to the LCD. "LCD ON BACKLIGHT CON" or insert
										\  "ON BACKLIGHT" into a section of code already
										\  bracketed by  "LCD ... CON"

: LCDHOME	( -- ) ." ?a" ;				\  "home" cursor (0,0)
: LCDCLS	( -- ) ." ?f" ;				\  clear screen, then home cursor
: LCDNL		( -- ) ." ?n" ;				\  go to start of next line

: LCDXY		( x y -- ) ." ?y" $010A .NUM ." ?x" $020A .NUM ;
										\  cursor to (x,y)
: LCDLIN0		( -- ) LCDCLS 9 0 LCDXY ;
: .duration		( -- ) duration @ $030A .NUM ;
: .onperiod		( -- ) onperiod @ $030A .NUM ;
: .offperiod	( -- ) offperiod @ $030A .NUM ;
: .runningtime	( -- ) runningtime @ $030A .NUM ;
: .totaltime	( -- ) totaltime @ $030A .NUM ;


:  EDITPARMS	( -- )
	LCDLIN0 .duration
	." ?n      duration "
	duration #maxdur QDEC@	
\ ---------------------------
	LCDLIN0 .onperiod
	." ?n      onperiod "
	onperiod #maxon QDEC@
\ ---------------------------
	LCDLIN0 .offperiod
	." ?n      offperiod "
	offperiod #maxoff QDEC@
	;

: SHOWPARMS	( -- )
	LCDCLS  2 0 LCDXY ." Parameter Values"
	." ?n    duration: " .duration
	." ?n    onperiod: " .onperiod
	." ?n   offperiod: " .offperiod
	;


: PUMPTASK	( -- )

		ON #MAIN GREEN LED			\  is assigned
		ON #PUMP GREEN LED			\  is assigned
		0 runningtime !				\  initialize
		0 totaltime !				\  initialize
		

			ON PUMP
			ON #PUMP RED LED			\  pump is actually running
			
			duration @ onperiod @ / 1+  FOR
				onperiod @ FOR
					#10 FOR
						6 seconds
						TANKFULL? IF	\ LOWBATTERY? also
							OFF PUMP
							OFF #PUMP RED LED
							ON #PUMP YELLOW	LED	\  PUMP NOT RUNNING DUE  FAULT
							2 seconds
							EXIT
						THEN
					NEXT
					1 runningtime +!
					1 totaltime +!	
				NEXT
				
				OFF #PUMP RED LED
				offperiod @ FOR		\  'rest' period between pumping sessions
					#10 FOR 6 seconds NEXT
					1 totaltime +!
				NEXT
				ON #PUMP RED LED
					
			NEXT
			OFF PUMP
			OFF #PUMP RED LED
			OFF #PUMP GREEN LED
			OFF #MAIN GREEN LED			\  is assigned
			12 seconds
			EXIT

		;
		
: .RUNTIME ( -- )
		3 2 LCDXY ." running:  " .runningtime
		3 3 LCDXY ."   total:  " .totaltime
		;
		
\  --------------------------------------------------------
: ABORT? TANKFULL?	( -- ) ;
\  --------------------------------------------------------
	
: RUNPUMP	( -- )  
LCD
		LCDCLS
		4 0 LCDXY .duration "/" EMIT .onperiod "/" EMIT .offperiod
		3 1 LCDXY ." Pump  Running" 
CON	
		' PUMPTASK  4 RUN
LCD	
		BEGIN
			.RUNTIME	4 seconds
			
			runningtime @ duration @ < IF	FALSE		ELSE
			ABORT?					   IF	TRUE		ELSE
					 						TRUE   THEN THEN
		UNTIL
		;

\  --------------------------------------------------------
:
 PUMPTIMER	( -- )		\ TOP LEVEL WORDS

\  ------------------ SetUp -----------------------------

	#19200 SERBAUD

	#60	duration !				\ SET DEFAULT VALUES
	#10 onperiod !
	#3  offperiod !

	LCD							\  redirect console to LCD
	
\  -------------------------------------------------------

	BEGIN					\  MAIN LOOP
		0 runningtime !		\  initialize
		0 totaltime !		\  initialize
		SHOWPARMS
		BEGIN
			SELECTED?	IF	EDITPARMS FALSE TRUE	ELSE	\  EDIT, NEEDS PARM REDISPLAY
			RUN?		IF	RUNPUMP FALSE TRUE		ELSE	\  RUN, NEEDS ALSO
			ABORT?		IF	TRUE TRUE				ELSE	\  ABORT, FALLS THRU BOTH UNTILS
							FALSE		  THEN THEN THEN	\  DEFAULT, COME AROUND AND TEST
							 								\  BUTTONS AGAIN		
			150 ms				\ slow down loop to give debounce a chance
		UNTIL
	UNTIL
	
\  ----------------------------------------------------------
	
	
	CON							\  return console
	;
\  ------------- Brian's shortcuts ----------------------------

: PP PUMPTIMER	( -- )  ;
: pp PUMPTIMER	( -- )  ;
: RT .RUNTIME	( -- )  ;
: rt BEGIN RT 29 seconds KEY? UNTIL  ; 

END
                  


	